from gnuradio import blocks
from gnuradio import eng_notation
from gnuradio import gr
from gnuradio import uhd
from gnuradio.eng_option import eng_option
from gnuradio.filter import firdes
from optparse import OptionParser
from ..base import ScopeTemplate
from chipwhisperer.common.utils import util
from chipwhisperer.common.utils.parameter import setupSetParam, Parameterized
from chipwhisperer.common.utils.pluginmanager import Plugin
from chipwhisperer.common.ui.ProgressBar import *
from time import sleep
from threading import Timer
import numpy as np
import time
import sys


class top_block(gr.top_block):

    def __init__(self):
        gr.top_block.__init__(self, "Top Block")

        ##################################################
        # Variables
        ##################################################
        self.samp_rate = samp_rate = 100000
        self.freq = freq = 80000000
        self.length = length = 2600
        self.gain = gain = 0

        ##################################################
        # Blocks
        ##################################################
        self.uhd_usrp_source_0 = uhd.usrp_source(  # Set recv_frame_size to 65536 and num_recv_frames to 1024
        	",".join(("", "recv_frame_size=4096", "num_recv_frames=1024")),
            #",".join(("", "")),
        	uhd.stream_args(
        		cpu_format="fc32",
        		channels=range(1),
        	),
        )
        logging.info("Special conf")
        self.uhd_usrp_source_0.set_samp_rate(samp_rate)
        self.uhd_usrp_source_0.set_center_freq(freq, 0)
        self.uhd_usrp_source_0.set_gain(gain, 0)
        self.uhd_usrp_source_0.set_min_output_buffer(16*1024*1024)  # 16 MB output buffer
        self.blocks_vector_sink_0 = blocks.vector_sink_c(length)
        self.blocks_stream_to_vector_0 = blocks.stream_to_vector(gr.sizeof_gr_complex, length)
        self.blocks_head_0 = blocks.head(gr.sizeof_gr_complex, length)

        ##################################################
        # Connections
        ##################################################
        self.connect((self.uhd_usrp_source_0, 0), (self.blocks_head_0, 0))
        self.connect((self.blocks_head_0, 0), (self.blocks_stream_to_vector_0, 0))
        self.connect((self.blocks_stream_to_vector_0, 0), (self.blocks_vector_sink_0, 0))

    def get_samp_rate(self):
        return self.samp_rate

    def set_samp_rate(self, samp_rate):
        self.samp_rate = samp_rate
        self.uhd_usrp_source_0.set_samp_rate(self.samp_rate)

    def get_freq(self):
        return self.freq

    def set_freq(self, freq):
        self.freq = freq
        self.uhd_usrp_source_0.set_center_freq(self.freq, 0)

    def get_gain(self):
        return self.gain

    def set_gain(self, gain):
        self.gain = gain
        self.uhd_usrp_source_0.set_gain(self.gain, 0)

    def get_length(self):
        return self.length

    def set_length(self, length):
        self.length = length

        self.lock()

        self.disconnect((self.uhd_usrp_source_0, 0), (self.blocks_head_0, 0))
        self.disconnect((self.blocks_head_0, 0), (self.blocks_stream_to_vector_0, 0))
        self.disconnect((self.blocks_stream_to_vector_0, 0), (self.blocks_vector_sink_0, 0))

        self.blocks_head_0 = blocks.head(gr.sizeof_gr_complex, length)
        self.blocks_stream_to_vector_0 = blocks.stream_to_vector(gr.sizeof_gr_complex, length)
        self.blocks_vector_sink_0 = blocks.vector_sink_c(length)

        self.connect((self.uhd_usrp_source_0, 0), (self.blocks_head_0, 0))
        self.connect((self.blocks_head_0, 0), (self.blocks_stream_to_vector_0, 0))
        self.connect((self.blocks_stream_to_vector_0, 0), (self.blocks_vector_sink_0, 0))

        self.unlock()


class USRPBase(Parameterized, Plugin):
    _name = 'USRP'

    def __init__(self):
        self.usrp = None
        self.dataUpdated = util.Signal()

        self.getParams().addChildren([
            {'name':'Sample Rate', 'key':'samplerate', 'type':'int', 'step':1E5, 'limits':(10000, 6E9), 'value':1E5, 'action':self.updateSettings, 'siPrefix':True, 'suffix': 'S/s'},
            {'name':'Frequency', 'key':'freq', 'type':'int', 'step':1E3, 'limits':(0, 6000000000), 'value':80000000, 'action':self.updateSettings, 'siPrefix':True, 'suffix': 'Hz'},
            {'name':'Gain', 'key':'gain', 'type':'int', 'step':1, 'limits':(0, 40), 'value':0, 'action':self.updateSettings, 'siPrefix':True, 'suffix': 'dB'},
            {'name':'Samples to capture', 'key':'length', 'type':'int', 'step':1, 'limits':(0, 6E10), 'value':2600, 'action':self.updateLen, 'siPrefix':True, 'suffix': 'samples'},
            {'name':'Samples to skip', 'key':'skip', 'type':'int', 'step':1, 'limits':(0, 6E10), 'value':0, 'action':self.updateSettings, 'siPrefix':True, 'suffix': 'samples'},
        ])


    @setupSetParam("")
    def updateSettings(self, _=None):
        if not self.usrp is None:
            self.usrp.set_samp_rate(self.findParam('samplerate').getValue())
            self.usrp.set_freq(self.findParam('freq').getValue())
            self.usrp.set_gain(self.findParam('gain').getValue())

    @setupSetParam("")
    def updateLen(self, _=None):
        if not self.usrp is None:
            self.usrp.set_length(self.findParam('length').getValue())

    def con(self):
        #logging.info("CON")
        self.usrp = top_block()

    def dis(self):
        #logging.info("DIS")
        if not self.usrp is None:
            self.usrp.stop()
            self.usrp.wait()
            self.usrp = None

    def arm(self):
        #logging.info("ARM")
        if not self.usrp is None:
            self.usrp.stop()
            self.usrp.wait()
            self.usrp.start()

    def timeout(self):
        logging.warning("Timeout on capture, skipping...")
        self.usrp.stop()

    def capture(self):
        #logging.info("CAP")
        channel = 0
        to_skip = self.findParam('skip').getValue()

        # Get data
        if not self.usrp is None:
            t = Timer(1.0, self.timeout)
            t.start()
            self.usrp.wait()
            t.cancel()
            data_complex = np.array(self.usrp.blocks_vector_sink_0.data())
            data = np.abs(data_complex[to_skip:])
            if len(data) == 0:
                data = [0] * self.findParam('length').getValue()
            self.usrp.stop()
            self.usrp.blocks_head_0.reset()
            self.usrp.blocks_vector_sink_0.reset()

            self.datapoints = data
            self.dataUpdated.emit(channel, self.datapoints, 0, self.usrp.samp_rate)

        # No timeout?
        return False

def main(top_block_cls=top_block, options=None):
    tb = top_block_cls()
    tb.start()
    try:
        raw_input('Press Enter to quit: ')
    except EOFError:
        pass
    tb.stop()
    tb.wait()


if __name__ == '__main__':
    main()
